"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Rx = require("rxjs");
exports.State = {
    OK: 'OK',
    ERROR: 'ERROR'
};
class QEntry {
    constructor(task) {
        this.executeCnt = 0;
        this.task = task;
        this.created = new Date();
    }
}
exports.QEntry = QEntry;
class Queue {
    constructor(logger, argv) {
        // public q: Rx.Observable<Entry>;
        this.qEntries = [];
        this.inProcess = false;
        this.taskTimer = argv.taskTimer || 500;
        this.reclaimTimeout = argv.reclaimTimeout || 10000;
        this.retryWaitTime = argv.retryWaitTime || 1000;
        this.maxExecuteCnt = argv.maxExecuteCnt || 10;
        this.logger = logger;
        this.q = new Rx.Subject();
        this.deadLetter = new Rx.Subject();
        this.q.subscribe(this.action.bind(this));
        this.qTask = setInterval(this.processMemoryQ.bind(this), this.taskTimer);
        //   next: (e) => {
        //     console.log('observerA: ' + e);
        //   }
        // });
    }
    stop() {
        this.logger.info('run stop');
        console.log('run stop');
        return Rx.Observable.create((observer) => {
            this.logger.info('queue stop');
            const action = () => {
                if (this.qEntries.length == 0) {
                    this.logger.info('Q Task stopped');
                    clearInterval(this.qTask);
                    observer.next(null);
                    observer.complete();
                }
                else {
                    this.logger.debug('waiting for Q Task to stop');
                    setTimeout(action, this.taskTimer);
                }
            };
            action();
        });
    }
    action(qe) {
        this.logger.debug('action:', qe);
        qe.running = new Date();
        ++qe.executeCnt;
        qe.task.subscribe((_qe) => {
            qe.running = null;
            qe.completed = new Date();
            this.logger.debug('action:completed:', qe);
        }, (error) => {
            qe.running = null;
            qe.retryAt = new Date((new Date()).getTime() + this.retryWaitTime);
            this.logger.debug('action:error:', qe.created.getTime(), qe.retryAt.getTime(), qe);
        });
    }
    processMemoryQ() {
        if (this.inProcess) {
            return;
        }
        this.inProcess = true;
        let now = (new Date()).getTime();
        this.qEntries = this.qEntries.filter((qe) => {
            if (qe.completed) {
                this.logger.debug('completed remove', qe);
                return false;
            }
            else if (qe.executeCnt >= this.maxExecuteCnt) {
                this.logger.error('drop queue entry:', qe);
                this.deadLetter.next(qe);
                return false;
            }
            else if (qe.retryAt) {
                if (now >= qe.retryAt.getTime()) {
                    qe.retryAt = null;
                    this.logger.debug('retry queue entry:', qe);
                    this.q.next(qe);
                }
                return true;
            }
            else if (!qe.running) {
                this.logger.debug('trigger action:', qe);
                this.q.next(qe);
            }
            else if ((qe.running && qe.executeCnt < this.maxExecuteCnt &&
                (now - qe.running.getTime()) >= this.reclaimTimeout)) {
                qe.running = null;
                this.logger.debug('retry execution time entry:', qe);
                this.q.next(qe);
            }
            return true;
        });
        this.inProcess = false;
    }
    push(action) {
        this.qEntries.push(new QEntry(action));
        this.processMemoryQ();
    }
}
exports.Queue = Queue;
function start(logger, argv) {
    return new Queue(logger, argv);
}
exports.start = start;
exports.default = Queue;
//# sourceMappingURL=simqle.js.map